package minischeme

import java.io.{FileOutputStream,PrintStream,FileNotFoundException}
import scala.util.parsing.input.Reader
import scala.collection.immutable.ListMap
import SymbolicTreeModule._

abstract class MainCommon {
  val globalEnv = ListMap(Primitives.symbols map { s => (s.name, s) } : _*)

  def usageString: String
  def error(msg: String): Nothing = {
    Console.println("Usage: scala minischeme." + usageString)
    Console.println("\nError: " + msg)
    exit(1)
  }

  def parseInputFiles(inFiles: List[String]): List[Tree] = {
    if (inFiles.isEmpty)
      error("no input file(s) given")

    try {
      val inReader: Reader[Char] = SeqReader(inFiles map { FileReader(_) })
      val parserModule = new SExpParsers
      val scanner = new parserModule.lexical.Scanner(inReader)
      parserModule.program(scanner) match {
        case parserModule.Success(program, _) =>
          EtaExpander(NameAnalyzer(program, globalEnv))
        case e: parserModule.NoSuccess =>
          Console.println(e)
          exit(1)
      }
    } catch {
      case e : FileNotFoundException => error(e.getMessage)
    }
  }
}

object MainInterpreter extends MainCommon {
  override def usageString: String = "MainInterpreter <input_file> ..."

  def main(args: Array[String]): Unit =
    Interpreter(parseInputFiles(args.toList))
}

object MainCompiler extends MainCommon {
  override def usageString: String =
    "MainCompiler [-o <output_file>] <input_file> ..."

  def main(args: Array[String]): Unit = {
    try {
      val (inFiles, outStream) = {
        def loop(args: List[String]): (List[String], PrintStream) =
          args match {
            case "-o" :: outFile :: rest =>
              (rest, new PrintStream(new FileOutputStream(outFile)))
            case arg :: rest =>
              val (in, out) = loop(rest)
              (arg :: in, out)
            case Nil =>
              (Nil, System.out)
          }
        loop(args.toList)
      }

      Generator(parseInputFiles(inFiles), outStream)
    } catch {
      case e: FileNotFoundException => error(e.getMessage)
    }
  }
}
