package minischeme

import scala.util.parsing.combinator.lexical.Scanners
import scala.util.parsing.input.CharArrayReader.EofCh

/**
 * Scanner for minischeme.
 *
 * @author Michel Schinz
 */

class SExpScanners extends Scanners with SExpTokens {
  override def whitespace: Parser[Any] =
    rep(anyCharIn(" \t\n\r") | (';' ~ rep(anyCharNotIn("\n\r"))))

  override def token: Parser[Token] = (
      identStart ~! rep(identStart | digit)
        ^^ { case l ~ ls => processIdent(l :: ls) }
    | '"' ~> rep(anyCharNotIn("\n\r\"" + EofCh)) <~ '"'
        ^^ { chars => Str(charsToString(chars)) }
    | '#' ~> '\\' ~> anyCharNotIn(String.valueOf(EofCh))
        ^^ Chr
    | rep1(digit)
        ^^ { chars => Num(Integer.parseInt(charsToString(chars))) }
    | '(' ^^^ LPar
    | ')' ^^^ RPar
    | EofCh ^^^ EOF
    | failure("illegal character"))

  private def digit: Parser[Char] =
    anyCharIn("0123456789")
  private def identStart: Parser[Char] =
    anyCharIn("abcdefghijklmnopqrstuvwxyz|!$%&*+-./:<=>?@^_~")

  private def anyCharIn(set: String): Parser[Char] =
    elem("any in {" + set + "}",
         { c: Char => (set indexOf Character.toLowerCase(c)) != -1 })
  private def anyCharNotIn(set: String): Parser[Char] =
    elem("any not in {" + set + "}",
         { c: Char => (set indexOf Character.toLowerCase(c)) == -1 })

  private def processIdent(chars: List[Char]): Token =
    charsToString(chars) match {
      case "define" => Define
      case "lambda" => Lambda
      case "let" => Let
      case "begin" => Begin
      case "if" => If
      case "and" => And
      case "or" => Or
      case ident => Ident(ident)
    }

  private def charsToString(chars: List[Char]): String =
    chars mkString ""
}
