package minischeme

abstract class SymbolLocation
case class FrameLocation(offset: Int) extends SymbolLocation
case class GlobalLocation(offset: Int) extends SymbolLocation

abstract class SymbolKind
case object PrimitiveKind extends SymbolKind
case object GlobalKind extends SymbolKind
case object LocalKind extends SymbolKind

class Symbol(val name: String, val kind: SymbolKind) {
  var location: SymbolLocation = _
  override def toString(): String = name
}

object Symbol {
  private val counters = scala.collection.mutable.HashMap[String,Int]()

  private def freshName(prefix: String): String = {
    val count = counters.getOrElse(prefix, 1)
    counters.put(prefix, count + 1)
    prefix + "_" + count
  }

  def freshLocal(prefix: String): Symbol =
    new Symbol(freshName(prefix), LocalKind)
}
