package minischeme

/**
 * Minischeme AST without "define" statement.
 */
trait TreeModule[Identifier] {
  trait Tree extends scala.util.parsing.input.Positional
  case class Define(name: Identifier, value: Tree) extends Tree
  case class Lambda(args: List[Identifier], body: Tree) extends Tree
  case class Let(name: Identifier, value: Tree, body: Tree) extends Tree
  case class Begin(body: List[Tree]) extends Tree
  case class And(cond1: Tree, cond2: Tree) extends Tree
  case class Or(cond1: Tree, cond2: Tree) extends Tree
  case class If(cond: Tree, thenp: Tree, elsep: Tree) extends Tree
  case class Apply(fun: Tree, args: List[Tree]) extends Tree
  case class Prim(sym: Symbol, args: List[Tree]) extends Tree
  case class Ident(ident: Identifier) extends Tree
  case class Num(value: Int) extends Tree
}

/**
 * The three variants actually appearing during compilation.
 */
object NominalTreeModule extends TreeModule[String]
object SymbolicTreeModule extends TreeModule[Symbol]

